#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "mfhdf.h"

int IterateMetadata(int32 fid, char *HDFEOS5FileName);

int ReadMetadata(int32 fid, char *MetadataName, char *HDFEOS5FileName);

int he2mr(char *HDFEOS2FileName, char *HDFEOS5FileName)
{
  int32 fid;

  fid = SDstart(HDFEOS2FileName, DFACC_READ);
  if (fid == -1)
  {
    fprintf(stderr, "Unable to open HDF-EOS file %s \n", HDFEOS2FileName);
    exit(1);
  }

  if (IterateMetadata(fid, HDFEOS5FileName)!=0)
  {
    fprintf(stderr, "Cannot iterate HDFEOS file.\n");
    exit(1);
  }

  if (SDend(fid)==-1)
  {
    fprintf(stderr,"Cannot close HDF-EOS file.\n");
    exit(1);
  }

  return 0;
}

int IterateMetadata(int32 fid, char *HDFEOS5FileName)
{
  intn i, j;
  char MetadataName[100];
  char MetadataNames[100][100]={"CoreMetadata", "COREMETADATA", "coremetadata",
                                "ArchiveMetadata", "ARCHIVEMETADATA", "archivemetadata",
                                "CollectionMetadata", "COLLECTIONMETADATA", "collectionmetadata"
                               };

  for (i = 0; i <= 9; i++)
  {
    ReadMetadata(fid, MetadataNames[i], HDFEOS5FileName);
    for (j = 0; j <= 9; j++)
    {
      sprintf(MetadataName, "%s.%d", MetadataNames[i], j);
      ReadMetadata(fid, MetadataName, HDFEOS5FileName);
    }
  }

  return 0;
}

int ReadMetadata(int32 fid, char *MetadataName, char *HDFEOS5FileName)
{
  intn status = 0;
  int32 AttributeIndex, DataType, TotalSize;
  char *Metadata, AttributeName[MAX_NC_NAME];

  /* Look for attribute */
  AttributeIndex = SDfindattr(fid, MetadataName);
  if (AttributeIndex == -1)
  {
    return (1);
  }
  /* Get size of attribute  */
  status = SDattrinfo(fid, AttributeIndex, AttributeName, &DataType, &TotalSize);
  if (status == -1)
  {
    return (1);
  }

  /* Allocate memory for attribute contents */
  Metadata = (char *)calloc(TotalSize, 1);
  if (Metadata == NULL)
  {
    fprintf(stderr, "Unable to allocate %d bytes for %s.\n", TotalSize, MetadataName);
    exit(1);
  }

  /* Read attribute */
  status = SDreadattr(fid, AttributeIndex, Metadata);
  if (status == -1)
  {
    free(Metadata);
    return (1);
  }

  /* Write to HDF-EOS5 instead of tempfile. */
  /*
  fprintf(TempFile, "%s", Metadata);
  */
  
  he5mw(HDFEOS5FileName, MetadataName, Metadata, TotalSize);

  free(Metadata);

  return 0;
}

