
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "hdf5.h"

int WriteMetadata(hid_t fid, char *MetadataName, char *Metadata, size_t TotalSize);

int WriteDataset(hid_t gid, char *MetadataName, char *Metadata, size_t TotalSize);

int he5mw(char *HDFEOS5FileName, char *MetadataName, char *Metadata, size_t TotalSize)
{
  hid_t fid;

  if ((fid=H5Fopen(HDFEOS5FileName, H5F_ACC_RDWR, H5P_DEFAULT))<0)
  {
    fprintf(stderr, "Cannot open HDFEOS5 file: %s.\n", HDFEOS5FileName);
    exit(1);
  }

  WriteMetadata(fid, MetadataName, Metadata, TotalSize);

  if (H5Fclose(fid) == -1)
  {
    fprintf(stderr,"Cannot close HDFEOS file %s.\n", HDFEOS5FileName);
    exit(1);
  }
  return 0;
}

int WriteMetadata(hid_t fid, char *MetadataName, char *Metadata, size_t TotalSize)
{ 
  hid_t gid;

  if ((gid=H5Gopen(fid, "HDFEOS INFORMATION"))==-1) 
  {
    fprintf(stderr,"Cannot open metadata group HDFEOS INFORMATION.\n");
    exit(1);
  }

  WriteDataset(gid, MetadataName, Metadata, TotalSize);

  if (H5Gclose(gid) == -1)
  {
    fprintf(stderr,"Cannot close group for HDF-EOS file\n");
    exit(1);
  }

  return 0;
}

int WriteDataset(hid_t gid, char *MetadataName, char *Metadata, size_t TotalSize)
{
  hid_t did, dtype, dspace;
  hsize_t dims[1];

  /*  Create new metadata. */
  dtype = H5Tcopy (H5T_C_S1);
  if ((H5Tset_size (dtype, TotalSize))<0)
  {
    fprintf(stderr, "Cannot set size.\n");
    exit(1);
  }
  dims[0] = 1;
  dspace = H5Screate_simple(1, dims, NULL);
  if ((did = H5Dcreate(gid, MetadataName, dtype, dspace, H5P_DEFAULT))<0)
  {
    fprintf(stderr, "Cannot create metadata.\n");
    exit(1);
  }

  /*  Write new metadata. */
  if (H5Dwrite(did, dtype, dspace, H5S_ALL, H5P_DEFAULT, Metadata) < 0 ) 
  {
    fprintf(stderr,"Cannot write to dataset %s.\n", MetadataName);
    exit(1);
  }

  /*  Close dataspace. */
  if (H5Sclose(dspace) == -1)
  {
    fprintf(stderr,"Cannot close dataspace for %s.\n", MetadataName);
    exit(1);
  }

  /*  Close datatype. */
  if (H5Tclose(dtype) == -1)
  {
    fprintf(stderr,"Cannot close datatype for %s.\n", MetadataName);
    exit(1);
  }

  /*  Close Dataset.  */
  if (H5Dclose(did) == -1)
  {
    fprintf(stderr,"Cannot close dataset for %s.\n", MetadataName);
    exit(1);
  }

  return 0;
}

